@@ -315,10 +315,14 @@
 //! [macro]: https://docs.serde.rs/serde_json/macro.json.html
 //! [`serde-json-core`]: https://japaric.github.io/serde-json-core/serde_json_core/
 
-#![doc(html_root_url = "https://docs.rs/serde_json/1.0.19")]
+#![doc(html_root_url = "https://docs.rs/serde_json/1.0.28")]
+#![cfg_attr(feature = "cargo-clippy", allow(renamed_and_removed_lints))]
 #![cfg_attr(feature = "cargo-clippy", deny(clippy, clippy_pedantic))]
 // Whitelisted clippy lints
-#![cfg_attr(feature = "cargo-clippy", allow(doc_markdown, needless_pass_by_value))]
+#![cfg_attr(
+    feature = "cargo-clippy",
+    allow(doc_markdown, needless_pass_by_value)
+)]
 // Whitelisted clippy_pedantic lints
 #![cfg_attr(feature = "cargo-clippy", allow(
 // Deserializer::from_str, into_iter
@@ -347,10 +351,10 @@
 
 #[macro_use]
 extern crate serde;
-extern crate dtoa;
-extern crate itoa;
 #[cfg(feature = "preserve_order")]
-extern crate linked_hash_map;
+extern crate indexmap;
+extern crate itoa;
+extern crate ryu;
 
 #[doc(inline)]
 pub use self::de::{from_reader, from_slice, from_str, Deserializer, StreamDeserializer};
