@@ -10,13 +10,12 @@ use error::Error;
 use serde::de::{self, Unexpected, Visitor};
 use serde::{Deserialize, Deserializer, Serialize, Serializer};
 use std::fmt::{self, Debug, Display};
-use std::i64;
 
-#[cfg(feature = "arbitrary_precision")]
-use dtoa;
 #[cfg(feature = "arbitrary_precision")]
 use itoa;
 #[cfg(feature = "arbitrary_precision")]
+use ryu;
+#[cfg(feature = "arbitrary_precision")]
 use serde::de::{IntoDeserializer, MapAccess};
 
 use de::ParserNumber;
@@ -64,10 +63,8 @@ impl Number {
     /// # #[macro_use]
     /// # extern crate serde_json;
     /// #
-    /// # use std::i64;
-    /// #
     /// # fn main() {
-    /// let big = i64::MAX as u64 + 10;
+    /// let big = i64::max_value() as u64 + 10;
     /// let v = json!({ "a": 64, "b": big, "c": 256.0 });
     ///
     /// assert!(v["a"].is_i64());
@@ -170,10 +167,8 @@ impl Number {
     /// # #[macro_use]
     /// # extern crate serde_json;
     /// #
-    /// # use std::i64;
-    /// #
     /// # fn main() {
-    /// let big = i64::MAX as u64 + 10;
+    /// let big = i64::max_value() as u64 + 10;
     /// let v = json!({ "a": 64, "b": big, "c": 256.0 });
     ///
     /// assert_eq!(v["a"].as_i64(), Some(64));
@@ -271,9 +266,7 @@ impl Number {
                 }
                 #[cfg(feature = "arbitrary_precision")]
                 {
-                    let mut buf = Vec::new();
-                    dtoa::write(&mut buf, f).unwrap();
-                    String::from_utf8(buf).unwrap()
+                    ryu::Buffer::new().format(f).to_owned()
                 }
             };
             Some(Number { n: n })
@@ -708,9 +701,7 @@ macro_rules! impl_from_unsigned {
                         { N::PosInt(u as u64) }
                         #[cfg(feature = "arbitrary_precision")]
                         {
-                            let mut buf = Vec::new();
-                            itoa::write(&mut buf, u).unwrap();
-                            String::from_utf8(buf).unwrap()
+                            itoa::Buffer::new().format(u).to_owned()
                         }
                     };
                     Number { n: n }
@@ -739,9 +730,7 @@ macro_rules! impl_from_signed {
                         }
                         #[cfg(feature = "arbitrary_precision")]
                         {
-                            let mut buf = Vec::new();
-                            itoa::write(&mut buf, i).unwrap();
-                            String::from_utf8(buf).unwrap()
+                            itoa::Buffer::new().format(i).to_owned()
                         }
                     };
                     Number { n: n }
