@@ -16,15 +16,12 @@ use std::str;
 use super::error::{Error, ErrorCode, Result};
 use serde::ser::{self, Impossible};
 
-use dtoa;
 use itoa;
+use ryu;
 
 #[cfg(feature = "arbitrary_precision")]
 use serde::Serialize;
 
-#[cfg(feature = "arbitrary_precision")]
-use number::{SERDE_STRUCT_FIELD_NAME, SERDE_STRUCT_NAME};
-
 /// A structure for serializing Rust values into JSON.
 pub struct Serializer<W, F = CompactFormatter> {
     writer: W,
@@ -461,19 +458,12 @@ where
         }
     }
 
-    #[cfg(not(feature = "arbitrary_precision"))]
-    #[inline]
-    fn serialize_struct(self, _name: &'static str, len: usize) -> Result<Self::SerializeStruct> {
-        self.serialize_map(Some(len))
-    }
-
-    #[cfg(feature = "arbitrary_precision")]
     #[inline]
     fn serialize_struct(self, name: &'static str, len: usize) -> Result<Self::SerializeStruct> {
-        if name == SERDE_STRUCT_NAME {
-            Ok(Compound::Number { ser: self })
-        } else {
-            self.serialize_map(Some(len))
+        match name {
+            #[cfg(feature = "arbitrary_precision")]
+            ::number::SERDE_STRUCT_NAME => Ok(Compound::Number { ser: self }),
+            _ => self.serialize_map(Some(len)),
         }
     }
 
@@ -820,7 +810,7 @@ where
             }
             #[cfg(feature = "arbitrary_precision")]
             Compound::Number { ref mut ser, .. } => {
-                if key == SERDE_STRUCT_FIELD_NAME {
+                if key == ::number::SERDE_STRUCT_FIELD_NAME {
                     try!(value.serialize(NumberStrEmitter(&mut *ser)));
                     Ok(())
                 } else {
@@ -1140,8 +1130,8 @@ where
         Err(key_must_be_a_string())
     }
 
-    fn serialize_char(self, _value: char) -> Result<()> {
-        Err(key_must_be_a_string())
+    fn serialize_char(self, value: char) -> Result<()> {
+        self.ser.serialize_str(&value.to_string())
     }
 
     fn serialize_bytes(self, _value: &[u8]) -> Result<()> {
@@ -1554,7 +1544,9 @@ pub trait Formatter {
     where
         W: io::Write,
     {
-        dtoa::write(writer, value).map(drop)
+        let mut buffer = ryu::Buffer::new();
+        let s = buffer.format(value);
+        writer.write_all(s.as_bytes())
     }
 
     /// Writes a floating point value like `-31.26e+12` to the specified writer.
@@ -1563,7 +1555,9 @@ pub trait Formatter {
     where
         W: io::Write,
     {
-        dtoa::write(writer, value).map(drop)
+        let mut buffer = ryu::Buffer::new();
+        let s = buffer.format(value);
+        writer.write_all(s.as_bytes())
     }
 
     /// Writes a number that has already been rendered to a string.
