@@ -8,8 +8,6 @@
 
 use std::borrow::Cow;
 use std::fmt;
-use std::i64;
-use std::io;
 use std::slice;
 use std::str;
 use std::vec;
@@ -25,11 +23,10 @@ use map::Map;
 use number::Number;
 use value::Value;
 
-#[cfg(feature = "arbitrary_precision")]
 use serde::de;
 
 #[cfg(feature = "arbitrary_precision")]
-use number::{NumberFromString, SERDE_STRUCT_FIELD_NAME};
+use number::NumberFromString;
 
 impl<'de> Deserialize<'de> for Value {
     #[inline]
@@ -111,44 +108,28 @@ impl<'de> Deserialize<'de> for Value {
                 Ok(Value::Array(vec))
             }
 
-            #[cfg(not(feature = "arbitrary_precision"))]
             fn visit_map<V>(self, mut visitor: V) -> Result<Value, V::Error>
             where
                 V: MapAccess<'de>,
             {
-                let mut values = Map::new();
-
-                while let Some((key, value)) = try!(visitor.next_entry()) {
-                    values.insert(key, value);
-                }
-
-                Ok(Value::Object(values))
-            }
-
-            #[cfg(feature = "arbitrary_precision")]
-            fn visit_map<V>(self, mut visitor: V) -> Result<Value, V::Error>
-            where
-                V: MapAccess<'de>,
-            {
-                let mut key = String::new();
-                let number = visitor.next_key_seed(NumberOrObject { key: &mut key })?;
-                match number {
-                    Some(true) => {
+                match visitor.next_key_seed(KeyClassifier)? {
+                    #[cfg(feature = "arbitrary_precision")]
+                    Some(KeyClass::Number) => {
                         let number: NumberFromString = visitor.next_value()?;
                         return Ok(Value::Number(number.value));
                     }
-                    None => return Ok(Value::Object(Map::new())),
-                    Some(false) => {}
+                    Some(KeyClass::Map(first_key)) => {
+                        let mut values = Map::new();
+
+                        values.insert(first_key, try!(visitor.next_value()));
+                        while let Some((key, value)) = try!(visitor.next_entry()) {
+                            values.insert(key, value);
+                        }
+
+                        Ok(Value::Object(values))
+                    }
+                    None => Ok(Value::Object(Map::new())),
                 }
-
-                let mut values = Map::new();
-
-                values.insert(key, try!(visitor.next_value()));
-                while let Some((key, value)) = try!(visitor.next_entry()) {
-                    values.insert(key, value);
-                }
-
-                Ok(Value::Object(values))
             }
         }
 
@@ -156,68 +137,6 @@ impl<'de> Deserialize<'de> for Value {
     }
 }
 
-struct WriterFormatter<'a, 'b: 'a> {
-    inner: &'a mut fmt::Formatter<'b>,
-}
-
-impl<'a, 'b> io::Write for WriterFormatter<'a, 'b> {
-    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
-        fn io_error<E>(_: E) -> io::Error {
-            // Value does not matter because fmt::Debug and fmt::Display impls
-            // below just map it to fmt::Error
-            io::Error::new(io::ErrorKind::Other, "fmt error")
-        }
-        let s = try!(str::from_utf8(buf).map_err(io_error));
-        try!(self.inner.write_str(s).map_err(io_error));
-        Ok(buf.len())
-    }
-
-    fn flush(&mut self) -> io::Result<()> {
-        Ok(())
-    }
-}
-
-impl fmt::Display for Value {
-    /// Display a JSON value as a string.
-    ///
-    /// ```rust
-    /// # #[macro_use]
-    /// # extern crate serde_json;
-    /// #
-    /// # fn main() {
-    /// let json = json!({ "city": "London", "street": "10 Downing Street" });
-    ///
-    /// // Compact format:
-    /// //
-    /// // {"city":"London","street":"10 Downing Street"}
-    /// let compact = format!("{}", json);
-    /// assert_eq!(compact,
-    ///     "{\"city\":\"London\",\"street\":\"10 Downing Street\"}");
-    ///
-    /// // Pretty format:
-    /// //
-    /// // {
-    /// //   "city": "London",
-    /// //   "street": "10 Downing Street"
-    /// // }
-    /// let pretty = format!("{:#}", json);
-    /// assert_eq!(pretty,
-    ///     "{\n  \"city\": \"London\",\n  \"street\": \"10 Downing Street\"\n}");
-    /// # }
-    /// ```
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        let alternate = f.alternate();
-        let mut wr = WriterFormatter { inner: f };
-        if alternate {
-            // {:#}
-            super::super::ser::to_writer_pretty(&mut wr, self).map_err(|_| fmt::Error)
-        } else {
-            // {}
-            super::super::ser::to_writer(&mut wr, self).map_err(|_| fmt::Error)
-        }
-    }
-}
-
 impl str::FromStr for Value {
     type Err = Error;
     fn from_str(s: &str) -> Result<Value, Error> {
@@ -1367,52 +1286,51 @@ impl<'de> serde::Deserializer<'de> for MapKeyDeserializer<'de> {
     }
 }
 
-#[cfg(feature = "arbitrary_precision")]
-struct NumberOrObject<'a> {
-    key: &'a mut String,
+struct KeyClassifier;
+
+enum KeyClass {
+    Map(String),
+    #[cfg(feature = "arbitrary_precision")]
+    Number,
 }
 
-#[cfg(feature = "arbitrary_precision")]
-impl<'a, 'de> DeserializeSeed<'de> for NumberOrObject<'a> {
-    type Value = bool;
+impl<'de> DeserializeSeed<'de> for KeyClassifier {
+    type Value = KeyClass;
 
     fn deserialize<D>(self, deserializer: D) -> Result<Self::Value, D::Error>
     where
         D: serde::Deserializer<'de>,
     {
-        deserializer.deserialize_any(self)
+        deserializer.deserialize_str(self)
     }
 }
 
-#[cfg(feature = "arbitrary_precision")]
-impl<'a, 'de> Visitor<'de> for NumberOrObject<'a> {
-    type Value = bool;
+impl<'de> Visitor<'de> for KeyClassifier {
+    type Value = KeyClass;
 
     fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
         formatter.write_str("a string key")
     }
 
-    fn visit_str<E>(self, s: &str) -> Result<bool, E>
+    fn visit_str<E>(self, s: &str) -> Result<Self::Value, E>
     where
         E: de::Error,
     {
-        if s == SERDE_STRUCT_FIELD_NAME {
-            Ok(true)
-        } else {
-            self.key.push_str(s);
-            Ok(false)
+        match s {
+            #[cfg(feature = "arbitrary_precision")]
+            ::number::SERDE_STRUCT_FIELD_NAME => Ok(KeyClass::Number),
+            _ => Ok(KeyClass::Map(s.to_owned())),
         }
     }
 
-    fn visit_string<E>(self, s: String) -> Result<bool, E>
+    fn visit_string<E>(self, s: String) -> Result<Self::Value, E>
     where
         E: de::Error,
     {
-        if s == SERDE_STRUCT_FIELD_NAME {
-            Ok(true)
-        } else {
-            *self.key = s;
-            Ok(false)
+        match s.as_str() {
+            #[cfg(feature = "arbitrary_precision")]
+            ::number::SERDE_STRUCT_FIELD_NAME => Ok(KeyClass::Number),
+            _ => Ok(KeyClass::Map(s)),
         }
     }
 }
